#include <stdio.h>
#include <stdlib.h>
#include <dos.h>

#define CLOCK_CONTROL   0x70
#define CLOCK_DATA      0x71

#define DOS_SET_DATE    0x2B
#define DOS_SET_TIME    0x2D

#define ToBinary(Value) \
    ((((Value) >> 4) & 0x0F)*10 + ((Value) & 0x0F ))

int     ReadRtcByte(int Address) {
    int     Result;

    disable();
    outportb(CLOCK_CONTROL,Address);
    Result = inportb(CLOCK_DATA);
    enable();
    return Result;
}

void    SynchronizeRealTimeClock(void) {
    while (!(ReadRtcByte(0x0A) & 0x80)) /* Wait for update in progress */
        ;
    while (ReadRtcByte(0x0A) & 0x80)    /* Wait for update to finish */
        ;
}

void    ReReadRealTimeClock(int *Year,int *Month,int *DayOfWeek,int *Day,
    int *Hour,int *Minute,int *Second) {

   *Second = ToBinary(ReadRtcByte(0x00));
   *Minute = ToBinary(ReadRtcByte(0x02));
   *Hour = ToBinary(ReadRtcByte(0x04));
   *DayOfWeek = ToBinary(ReadRtcByte(0x06));
   *Day = ToBinary(ReadRtcByte(0x07));
   *Month = ToBinary(ReadRtcByte(0x08));
   *Year = ToBinary(ReadRtcByte(0x09));
    if (*Year >= 80)
        *Year += 1900;
    else
        *Year += 2000;
}

void    ReadRealTimeClock(int *Year,int *Month,int *DayOfWeek,int *Day,
    int *Hour,int *Minute,int *Second) {

    SynchronizeRealTimeClock();
    ReReadRealTimeClock(Year,Month,DayOfWeek,Day,Hour,Minute,Second);
}

int     SetMsDosTime(int Hour,int Minute,int Second,int Hundredths) {
    union REGS  Regs;

    Regs.h.ah = DOS_SET_TIME;
    Regs.h.ch = Hour;
    Regs.h.cl = Minute;
    Regs.h.dh = Second+1;   /* for some reason needed */
    Regs.h.dl = Hundredths;
    intdos(&Regs,&Regs);
    return Regs.h.al;
}

int     SetMsDosDate(int Year,int Month,int Day) {
    union REGS  Regs;

    Regs.h.ah = DOS_SET_DATE;
    Regs.x.cx = Year;
    Regs.h.dh = Month;
    Regs.h.dl = Day;
    intdos(&Regs,&Regs);
    return Regs.h.al;
}

int     main(void) {
    int     Year, Month, DayOfWeek, Day, Hour, Minute, Second, SaveSecond;
    int     Result;

    Result = EXIT_SUCCESS;
    do {
        ReadRealTimeClock(&Year,&Month,&DayOfWeek,&Day,&Hour,&Minute,&Second);
    } while (Second == 59);
    if (SetMsDosTime(Hour,Minute,Second,0)) {
        fprintf(stderr,"Set time failed.\n");
        Result = EXIT_FAILURE;
    } else if (SetMsDosDate(Year,Month,Day)) {
        fprintf(stderr,"Set date failed.\n");
        Result = EXIT_FAILURE;
    }
    SaveSecond = Second;
    ReReadRealTimeClock(&Year,&Month,&DayOfWeek,&Day,&Hour,&Minute,&Second);
    if (SaveSecond != Second) {
        fprintf(stderr,"Set time changed seconds.\n");
        Result = EXIT_FAILURE;
    }
    printf("%02d/%02d/%04d %02d:%02d:%02d",Month,Day,Year,Hour,Minute,Second);
    return Result;
}
