#include <stdio.h>  /* fprintf              */
#include <dos.h>    /* inportb, outportb    */
#include <bios.h>   /* _bios_serialcom      */

#ifdef __TURBOC__
#define InPortB(P)      inportb(P)
#define OutPortB(P,D)   outportb(P,D)
#else
#define InPortB(P)      inp(P)
#define OutPortB(P,D)   outp(P,D)
#endif

#define BIOS_DATA   ((int far *)(0x400000L))

#define IER     (ComPort+1) /* Interrupt Enable Register    */
#define IIR     (ComPort+2) /* Interrupt Identification     */
#define LCR     (ComPort+3) /* Line Control Register        */
#define MCR     (ComPort+4) /* Modem Control Register       */
#define LSR     (ComPort+5) /* Line Status Register         */
#define MSR     (ComPort+6) /* Modem Status Register        */

#define DATA    ComPort
#define STATUS  LSR

#define TRANSMIT_EMPTY  0x40
#define RECEIVE_HOLDING 0x01

#define ReportError(s)  fprintf(stderr,s)

static int  PortNumber;
static int  ComPort;

int     SSetup(short PortNumber,unsigned ComParams,unsigned ControlLines) {
    int     Result;

    Result = 0;

    if (PortNumber < 0 || PortNumber > 1)
        ReportError("Invalid port number.\n");
    else if ((ComPort = BIOS_DATA[PortNumber]) == 0)
        ReportError("BIOS could not find port.\n");
    else {
        _bios_serialcom(_COM_INIT,PortNumber,ComParams);
        OutPortB(MCR,ControlLines); /* Set up Modem Control Register    */
        Result = 1;
    }
    return Result;
}

int     SClose(void) {
    return 1;
}

int     SSendChar(int Ch) {
    int     Status;
    int     Result;

    Result = 0;

    Status = InPortB(STATUS);
    if (Status & TRANSMIT_EMPTY) {
        OutPortB(DATA,Ch);
        Result = 1;
    }
    return Result;
}

int     SRcvChar(void) {
    int     Status;
    long    StartTime, CurrentTime;
    int     TimeChanges;
    int     Result;

    Result = -1;

    StartTime = biostime(0,0);
    TimeChanges = 0;
    do {
        Status = InPortB(STATUS);
        CurrentTime = biostime(0,0);
        if (CurrentTime != StartTime) {
            StartTime = CurrentTime;
            TimeChanges++;
        }
        /* 55 ms <= Time Wait <= 110 ms */
    } while ((Status & RECEIVE_HOLDING) == 0 && TimeChanges < 2);
    if (Status & RECEIVE_HOLDING)
        Result = InPortB(DATA) & 0x7F;
    return Result;
}
